@@ -52,6 +52,7 @@ int mxt_new(struct libmaxtouch_ctx **ctx)
   new_ctx->log_level = LOG_ERROR;
   new_ctx->query = false;
   new_ctx->log_fn = mxt_log_stderr;
+  new_ctx->i2c_block_size = I2C_DEV_MAX_BLOCK;
 
   *ctx = new_ctx;
 
@@ -113,7 +114,6 @@ int mxt_scan(struct libmaxtouch_ctx *ctx, struct mxt_conn_info **conn,
   return ret;
 }
 
-
 //******************************************************************************
 /// \brief Create connection object
 /// \return #mxt_rc
@@ -132,7 +132,8 @@ int mxt_new_conn(struct mxt_conn_info **conn, enum mxt_device_type type)
 
 //******************************************************************************
 /// \brief Take a reference to the connection object
-struct mxt_conn_info *mxt_ref_conn(struct mxt_conn_info *conn) {
+struct mxt_conn_info *mxt_ref_conn(struct mxt_conn_info *conn)
+{
   if (conn == NULL)
     return NULL;
 
@@ -142,7 +143,8 @@ struct mxt_conn_info *mxt_ref_conn(struct mxt_conn_info *conn) {
 
 //******************************************************************************
 /// \brief Free connection object
-struct mxt_conn_info *mxt_unref_conn(struct mxt_conn_info *conn) {
+struct mxt_conn_info *mxt_unref_conn(struct mxt_conn_info *conn)
+{
   if (conn == NULL)
     return NULL;
 
@@ -829,6 +831,26 @@ int mxt_bootloader_read(struct mxt_device *mxt, unsigned char *buf, int count)
   return ret;
 }
 
+//******************************************************************************
+/// \brief Write to bootloader
+/// \return #mxt_rc
+static int i2c_dev_bootloader_write_blks(struct mxt_device *mxt, unsigned char const *buf, int count)
+{
+  int ret;
+  size_t received;
+  int off = 0;
+
+  while (off < count) {
+    ret = i2c_dev_bootloader_write(mxt, buf + off, count - off, &received);
+    if (ret)
+      return ret;
+
+    off += received;
+  }
+
+  return MXT_SUCCESS;
+}
+
 //******************************************************************************
 /// \brief Write to bootloader
 /// \return #mxt_rc
@@ -844,7 +866,7 @@ int mxt_bootloader_write(struct mxt_device *mxt, unsigned char const *buf, int c
 #endif /* HAVE_LIBUSB */
 
   case E_I2C_DEV:
-    ret = i2c_dev_bootloader_write(mxt, buf, count);
+    ret = i2c_dev_bootloader_write_blks(mxt, buf, count);
     break;
 
   case E_HIDRAW:
