@@ -46,6 +46,7 @@
 
 #define BUF_SIZE 1024
 
+
 //******************************************************************************
 /// \brief Initialize mXT device and read the info block
 /// \return #mxt_rc
@@ -103,6 +104,7 @@ static void print_usage(char *prog_name)
           "  --self-cap-tune-config     : tune self capacitance settings to config\n"
           "  --self-cap-tune-nvram      : tune self capacitance settings to NVRAM\n"
           "  --version                  : print version\n"
+          "  --block-size BLOCKSIZE     : set the maximum block size used for i2c transfers (default %d)\n"
           "\n"
           "Configuration file commands:\n"
           "  --load FILE                : upload cfg from FILE in .xcfg or OBP_RAW format\n"
@@ -145,6 +147,8 @@ static void print_usage(char *prog_name)
           "  --self-cap-signals         : capture self cap signals\n"
           "  --self-cap-deltas          : capture self cap deltas\n"
           "  --self-cap-refs            : capture self cap references\n"
+          "  --active-stylus-deltas     : capture active stylus deltas\n"
+          "  --active-stylus-refs       : capture active stylus references\n"
           "\n"
           "Device connection options:\n"
           "  -q [--query]               : scan for devices\n"
@@ -159,7 +163,7 @@ static void print_usage(char *prog_name)
           "\n"
           "Debug options:\n"
           "  -v [--verbose] LEVEL       : set debug level\n",
-          MXT_VERSION, prog_name);
+          MXT_VERSION, prog_name, I2C_DEV_MAX_BLOCK);
 }
 
 //******************************************************************************
@@ -173,7 +177,6 @@ int main (int argc, char *argv[])
   uint8_t backup_cmd = BACKUPNV_COMMAND;
   unsigned char self_test_cmd = SELF_TEST_ALL;
   uint16_t address = 0;
-  uint16_t object_address = 0;
   uint16_t count = 0;
   struct mxt_conn_info *conn = NULL;
   uint16_t object_type = 0;
@@ -184,8 +187,9 @@ int main (int argc, char *argv[])
   uint8_t t37_mode = DELTAS_MODE;
   bool format = false;
   uint16_t port = 4000;
+  int i2c_block_size = I2C_DEV_MAX_BLOCK;
   uint8_t t68_datatype = 1;
-  unsigned char databuf[BUF_SIZE];
+  unsigned char databuf;
   char strbuf2[BUF_SIZE];
   char strbuf[BUF_SIZE];
   strbuf[0] = '\0';
@@ -197,6 +201,7 @@ int main (int argc, char *argv[])
 
     static struct option long_options[] = {
       {"backup",           optional_argument, 0, 0},
+      {"block-size",       required_argument, 0, 0},
       {"bootloader-version", no_argument,     0, 0},
       {"bridge-client",    required_argument, 0, 'C'},
       {"calibrate",        no_argument,       0, 0},
@@ -229,6 +234,8 @@ int main (int argc, char *argv[])
       {"self-cap-signals", no_argument,       0, 0},
       {"self-cap-deltas",  no_argument,       0, 0},
       {"self-cap-refs",    no_argument,       0, 0},
+      {"active-stylus-deltas",  no_argument,       0, 0},
+      {"active-stylus-refs",    no_argument,       0, 0},
       {"bridge-server",    no_argument,       0, 'S'},
       {"test",             optional_argument, 0, 't'},
       {"type",             required_argument, 0, 'T'},
@@ -271,12 +278,12 @@ int main (int argc, char *argv[])
         if (cmd == CMD_NONE) {
           cmd = CMD_BACKUP;
           if (optarg) {
-            ret = mxt_convert_hex(optarg, &databuf[0], &count, sizeof(databuf));
+            ret = mxt_convert_hex(optarg, &databuf, &count, sizeof(databuf));
             if (ret || count == 0) {
               fprintf(stderr, "Hex convert error\n");
               ret = MXT_ERROR_BAD_INPUT;
             }
-            backup_cmd = databuf[0];
+            backup_cmd = databuf;
           }
         } else {
           print_usage(argv[0]);
@@ -379,6 +386,12 @@ int main (int argc, char *argv[])
         t37_mode = SELF_CAP_REFS;
       } else if (!strcmp(long_options[option_index].name, "self-cap-deltas")) {
         t37_mode = SELF_CAP_DELTAS;
+      } else if (!strcmp(long_options[option_index].name, "active-stylus-deltas")) {
+        t37_mode = AST_DELTAS;
+      } else if (!strcmp(long_options[option_index].name, "active-stylus-refs")) {
+        t37_mode = AST_REFS;
+      } else if (!strcmp(long_options[option_index].name, "block-size")) {
+        i2c_block_size = atoi(optarg);
       } else if (!strcmp(long_options[option_index].name, "version")) {
         printf("mxt-app %s%s\n", MXT_VERSION, ENABLE_DEBUG ? " DEBUG":"");
         return MXT_SUCCESS;
@@ -560,12 +573,12 @@ int main (int argc, char *argv[])
     case 't':
       if (cmd == CMD_NONE) {
         if (optarg) {
-          ret = mxt_convert_hex(optarg, &databuf[0], &count, sizeof(databuf));
+          ret = mxt_convert_hex(optarg, &databuf, &count, sizeof(databuf));
           if (ret) {
             fprintf(stderr, "Hex convert error\n");
             ret = MXT_ERROR_BAD_INPUT;
           } else {
-            self_test_cmd = databuf[0];
+            self_test_cmd = databuf;
           }
         }
         cmd = CMD_TEST;
@@ -614,6 +627,12 @@ int main (int argc, char *argv[])
   /* Debug does not work until mxt_set_verbose() is called */
   mxt_info(ctx, "Version:%s", MXT_VERSION);
 
+  /* Update the i2c block size */
+  if (i2c_block_size != I2C_DEV_MAX_BLOCK) {
+    mxt_verb(ctx, "Setting i2c_block_size from %d to %d", ctx->i2c_block_size, i2c_block_size);
+    ctx->i2c_block_size = i2c_block_size;
+  }
+
   if (cmd == CMD_WRITE || cmd == CMD_READ) {
     mxt_verb(ctx, "instance:%u", instance);
     mxt_verb(ctx, "count:%u", count);
@@ -638,43 +657,10 @@ int main (int argc, char *argv[])
   switch (cmd) {
   case CMD_WRITE:
     mxt_verb(ctx, "Write command");
-
-    if (object_type > 0) {
-      object_address = mxt_get_object_address(mxt, object_type, instance);
-      if (object_address == OBJECT_NOT_FOUND) {
-        fprintf(stderr, "No such object\n");
-        ret = MXT_ERROR_OBJECT_NOT_FOUND;
-        break;
-      }
-
-      mxt_verb(ctx, "T%u address:%u offset:%u", object_type,
-               object_address, address);
-      address = object_address + address;
-
-      if (count == 0) {
-        count = mxt_get_object_size(mxt, object_type);
-      }
-    } else if (count == 0) {
-      fprintf(stderr, "Not enough arguments!\n");
-      ret = MXT_ERROR_BAD_INPUT;
+    ret = mxt_handle_write_cmd(mxt, object_type, count, instance, address,
+                               argc, argv);
+    if (ret == MXT_ERROR_BAD_INPUT)
       goto free;
-    }
-
-    if (optind != (argc - 1)) {
-      fprintf(stderr, "Must give hex input\n");
-      ret = MXT_ERROR_BAD_INPUT;
-      goto free;
-    }
-
-    ret = mxt_convert_hex(argv[optind], databuf, &count, sizeof(databuf));
-    if (ret || count == 0) {
-      fprintf(stderr, "Hex convert error\n");
-      ret = MXT_ERROR_BAD_INPUT;
-    } else {
-      ret = mxt_write_register(mxt, databuf, address, count);
-      if (ret)
-        fprintf(stderr, "Write error\n");
-    }
     break;
 
   case CMD_READ:
